// 菊地文雄, 山本昌宏「微分方程式と計算機演習」, 山海堂 (1991), p. 39
/* ** JACOBI OR SOR METHOD **
   N : NUMBER OF UNKNOWS
   A : LOWER DIAGONAL
   F : KNOWN VECTOR & SOLUTION VECTOR
   X : SOLUTION VECTOR
   N : MAXIMUM NO. OF ITERATIONS
   EPS : THRESHOLD VALUE FOR ERROR
   METHOD : METHOD NUMBER
   OMEGA : RELAXATION PARAMETER
*/

#include <stdio.h>
#include <stdlib.h> // exit()
#include <math.h>   // fabs()
#define NDIM (100)

void iter(int ndim,
      double a[][ndim], double f[], double x[], double xnew[],
      double eps, double omega, int n, int m, int method, int *niter);

int main(void)
{
  int n, m, method, i,j,niter;
  double eps, omega;
  double a[NDIM][NDIM], f[NDIM], x[NDIM], xnew[NDIM];
  printf("INPUT : N=\n");
  scanf("%d", &n);
  if (n <= 1) exit(1);
  printf("INPUT : M=\n");
  scanf("%d", &m);
  printf("INPUT : EPS=\n");
  scanf("%lf", &eps);
  printf("JACOBI : METHOD=0, SOR : =1\n");
  printf("INPUT : METHOD=\n");
  scanf("%d", &method);
  omega=1.0;
  if (method == 1) {
    printf("INPUT : OMEGA=\n");
    scanf("%lf", &omega);
  }
  for (i=0; i<n; i++)
    for (j=0; j<n; j++) {
      printf("INPUT : A(%3d,%3d)=\n", i+1, j+1);
      scanf("%lf", &a[i][j]);
    }
  for (i=0; i<n; i++) {
    printf("INPUT : F(%3d)=\n", i+1);
    scanf("%lf", &f[i]);
  }
  for (i=0; i<n; i++) {
    printf("INPUT : X(%3d)=\n", i+1);
    scanf("%lf", &x[i]);
  }
  // 試しに表示する
  printf("n=%d, m=%d, eps=%f, method=%d\n", n, m, eps, method);
  for (i=0; i<n; i++) {
    for (j=0; j<n; j++)
      printf("%f ", a[i][j]);
    printf("   %f %f", f[i], x[i]);
    printf("\n");
  }
  iter(NDIM,a,f,x,xnew,eps,omega,n,m,method,&niter);
  printf("NO. OF ITERATIONS=%3d   EPS=%12.4e\n", niter, eps);
  for (i = 0; i < 5; i++)
    printf("  I     X(I)    ");
  printf("\n");
  for (i=0; i<n; i++) {
    printf("%3d%12.4e ", i+1, x[i]);
    if ((i+1) % 5 == 0 || (i+1) == n)
      printf("\n");
  }
}

void iter(int ndim,
      double a[][ndim], double f[], double x[], double xnew[],
      double eps, double omega, int n, int m, int method, int *niter)
{
  int i, j;
  double error, xmax, xtemp, dx;
  *niter = 0;
  do {
    error = 0.0;
    xmax = 0.0;
    for (i=0; i<n; i++) {
      xtemp = f[i];
      for (j=0; j<n; j++)
    if (j != i) xtemp -= a[i][j] * x[j];
      xtemp /= a[i][i];
      if (method == 1)
    xtemp = omega * xtemp + (1.0 - omega) * x[i];
      dx = fabs(xtemp - x[i]);
      if (dx > error) error = dx;
      if (fabs(xtemp) >  xmax) xmax = fabs(xtemp);
      if (method == 0) xnew[i] = xtemp;
      if (method == 1) x[i] = xtemp;
    }
    if (method == 0)
      for (i=0; i<n; i++) x[i] = xnew[i];
    (*niter)++;
    error /= xmax;
    printf("(NITER,ERROR)=%d %g\n", *niter, error);
    if (*niter == m)
      return;
  } while (error > eps);
}
