// 菊地文雄, 山本昌宏「微分方程式と計算機演習」, 山海堂 (1991), p. 58
/* ** FINITE DIFFERENCE METHOD **
   **      FOR -D2U/Dx2=F      **
   N : NUMBER OF DIVISIONS
   H : MESH SIZE
   A,B,C : COEFFICIENT MATRIX
   F : FREE TERM & SOLUTION VECTOR
*/

#include <stdio.h>
#include <stdlib.h>
#define NDIM (100)

void trid(double a[], double b[], double c[], double f[], int n);
  
int main(void)
{
  int n, i;
  double h, h2;
  double a[NDIM], b[NDIM], c[NDIM], f[NDIM], x[NDIM];
  printf("INPUT : N=\n");
  scanf("%d", &n);
  if (n <= 1) exit(0);
  h = 1.0 / n;
  h2 = h * h;
  for (i=0; i<n; i++) {
    a[i] = -1.0 / h2;
    b[i] = 2.0 / h2;
    c[i] = -1.0 / h2;
    f[i] = 1.0;
  }
  b[n-1] = 1.0 / h2;
  f[n-1] = 0;
  trid(a,b,c,f,n);
  printf("MESH SIZE=%12.4e\n", h);
  printf("** NODAL VALUES OF SOLUTION **\n");
  for (i = 0; i < 5; i++)
    printf("  I     U(I)    ");
  printf("\n");
  for (i=0; i<n; i++) {
    printf("%3d%12.4e ", i+1, f[i]);
    if ((i+1) % 5 == 0 || (i+1) == n)
    printf("\n");
  }
}

void trid(double a[], double b[], double c[], double f[], int n)
{
  int i;
  double aa;
  // FORWARD ELIMITNATION
  for (i = 0; i < n-1; i++) {
    aa = a[i+1] / b[i];
    b[i+1] -= aa * c[i];
    f[i+1] -= aa * f[i];
  }
  // BACKWARD SUBSTITUTION
  f[n-1] /= b[n-1];
  for (i=n-2; i>= 0; i--)
    f[i] = (f[i] - c[i] * f[i+1]) / b[i];
}
