// 菊地文雄, 山本昌宏「微分方程式と計算機演習」, 山海堂 (1991), p. 65
/* ** FINITE DIFFERENCE METHOD **
   ** FOR -FNU D2U/Dx2+DU/DX=F **
   FNU : DIFFUSIVITY
   N : NO. OF DIVISIONS
   METHOD : METHOD NUMBER
   H : MESH SIZE
   A,B,C : COEFFICIENT MATRIX
   F : FREE TERM & SOLUTION VECTOR
*/

#include <stdio.h>
#include <stdlib.h>
#define NDIM (100)
void trid(double a[], double b[], double c[], double f[], int n);
  
int main(void)
{
  int n, i, method;
  double fnu, h, h2; // fnu は nu でも良いかも。
  double a[NDIM], b[NDIM], c[NDIM], f[NDIM], x[NDIM];
  printf("INPUT : FNU=\n");
  scanf("%lf", &fnu);
  printf("INPUT : N=\n");
  scanf("%d", &n);
  if (n <= 1) exit(1);
  printf("BACKWARD : METHOD=1, CENTRAL : =2, FORWARD : =3\n");
  printf("INPUT : METHOD=\n");
  scanf("%d", &method);
  h = 1.0 / n; // (double) n とする必要もないでしょう。
  h2 = h * h;  // Cに冪乗演算子はないので、乗算で代用
  for (i=0; i<n-1; i++) {
    a[i] = - fnu / h2;
    b[i] = 2.0 * fnu / h2;
    c[i] = - fnu / h2;
    if (method == 1) {
      // BACKWARD
      a[i] -= 1.0 / h;
      b[i] += 1.0 / h;
    }
    else if (method == 2) {
      // CENTRAL
      a[i] -= 0.5 / h;
      c[i] += 0.5 / h;
    }
    else {
      // FORWARD
      b[i] -= 1.0 / h;
      c[i] += 1.0 / h;
    }
    f[i] = 1.0;
  }
  trid(a,b,c,f,n-1);
  printf("DIFFUSIVITY=%12.4e   MESH SIZE=%12.4e   METHOD=%2d\n",
     fnu, h, method);
  printf("** NODAL VALUES OF SOLUTION **\n");
  for (i = 0; i < 5; i++)
    printf("  I     U(I)    ");
  printf("\n");
  for (i=0; i<n-1; i++) {
    printf("%3d%12.4e ", i+1, f[i]);
    if ((i+1) % 5 == 0 || (i+1) == n-1)
    printf("\n");
  }
}

void trid(double a[], double b[], double c[], double f[], int n)
{
  int i;
  double aa;
  // FORWARD ELIMITNATION
  for (i=0; i<n-1; i++) {
    aa = a[i+1] / b[i];
    b[i+1] -= aa * c[i];
    f[i+1] -= aa * f[i];
  }
  // BACKWARD SUBSTITUTION
  f[n-1] /= b[n-1];
  for (i=n-2; i>= 0; i--)
    f[i] = (f[i] - c[i]*f[i+1]) / b[i];
}
