// 菊地文雄, 山本昌宏「微分方程式と計算機演習」, 山海堂 (1991), p. 73
/*  ** FINITE DIFFERENCE METHOD **
    **      FOR -D2U/Dx2=F      **
    ** BY JACOBI OR SOR METHOD  **
    N : NO. OF DIVISIONS
    M : MAXIMUM NO. OF ITERATIONS
    EPS : THRESHOLD VALUE FOR ERROR
    METHOD : METHOD NUMBER
    OMEGA : RELAXATION PARAMETER
    H : MESH SIZE
    F : FREE TERM
    U : SOLUTION VECTOR
*/

#include <stdio.h>
#include <stdlib.h>  // exit()
#include <math.h>    // fabs()

#define NDIM (100)

void iter(double f[], double u[], double eps, double h2,
      double omega, int N, int M, int METHOD,
      int *niter);

int main(void)
{
  int n, m, method, i, niter;
  double eps, omega, h, h2;
  double f[NDIM],u[NDIM];
  printf("INPUT : N=\n");
  scanf("%d", &n);
  if (n <= 1) exit(1);
  printf("INPUT : M=\n");
  scanf("%d", &m);
  printf("INPUT : EPS=\n");
  scanf("%lf", &eps);
  printf("JACOBI : METHOD=0, SOR : =1\n");
  printf("INPUT : METHOD=\n");
  scanf("%d", &method);
  if (method == 1) {
    printf("INPUT : OMEGA=\n");
    scanf("%lf", &omega);
  }
  h = 1.0 / n;
  h2 = h * h;
  for (i = 0; i < n; i++) {
    u[i] = 0.0;
    f[i] = 1.0;
  }
  f[n-1] = 0.0;
  iter(f, u, eps, h2, omega, n, m, method, &niter);
  printf("NO. OF ITERATIONS=%4d   EPS=%12.4e\n", niter, eps);
  printf("MESH SIZE=%12.4e\n", h);
  printf("** NODAL VALUES OF SOLUTION **\n");
  for (i = 0; i < 5; i++)
    printf("  I     U(I)    ");
  printf("\n");
  for (i=0; i<n; i++) {
    printf("%3d%12.4e ", i+1, u[i]);
    if ((i+1) % 5 == 0 || (i+1) == n)
      printf("\n");
  }
}

void iter(double f[], double u[], double eps, double h2,
      double omega, int n, int m, int method,
      int *niter)
{
  int i;
  double error, umax, uleft, unew, du;
  for (i = 0; i < n; i++)
    printf("%f %f\n", u[i], f[i]);
  *niter = 0;
  do {
    error = 0.0;
    umax = 0.0;
    uleft = 0.0;
    for (i = 0; i < n; i++) {
      if (i != n-1) unew = (h2 * f[i] + uleft + u[i+1]) / 2.0;
      if (i == n-1) unew = uleft;
      if (method == 1) unew = omega * unew + (1.0 - omega) * u[i];
      du = fabs(unew - u[i]);
      if (du > error) error = du;
      if (fabs(unew) > umax) umax = fabs(unew);
      if (method == 0) uleft = u[i];
      if (method == 1) uleft = unew;
      u[i] = unew;
    }
    (*niter)++;
    error /= umax;
    printf("(NITER,ERROR)=%d %g\n", *niter, error);
    if (*niter == m) return;
  }
  while (error > eps);
}
