// 菊地文雄, 山本昌宏「微分方程式と計算機演習」, 山海堂 (1991), p. 88
/* ** POWER METHOD **
   N : ORDER OF MATRIX
   A : COEFFICIENT MATRIX
   EPS : THRESHOLD VALUE FOR ERRORS
   M : MAXIMUM NO. OF ITERATIONS
   EVAL : EIGENVALUE
   X : EIGENVECTOR
   NITER : NO. OF ITERATIONS
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h> // sqrt(),fabs()
#define NDIM (100)

void mpr(int ndim, double a[][ndim], double x[], int n, double y[]);
void inpr(double x[], double y[], int n, double *s);

int main(void)
{
  int n, m, i, j, niter;
  double eps, eval, s, enew, error;
  double a[NDIM][NDIM], x[NDIM], y[NDIM];
  // INPUT
  printf("INPUT : N=\n");
  scanf("%d", &n);
  if (n <= 1) exit(1);
  printf("INPUT : EPS\n");
  scanf("%lf", &eps);
  printf("INPUT : M=\n");
  scanf("%d", &m);
  for (i = 0; i < n; i++)
    for (j = 0; j < n; j++) {
      printf("INPUT : A(%3d,%3d)=\n", i+1, j+1);
      scanf("%lf", &a[i][j]);
    }
  for (i = 0; i < n; i++) {
    printf("INPUT : X(%3d)=\n", i);
    scanf("%lf", &x[i]);
  }
  // ITERATION
  niter = 0;
  eval = 0.0;
  mpr(NDIM,a,x,n,y);
  do {
    inpr(y,y,n,&s);
    s=sqrt(s);
    for (i=0; i<n; i++)
      x[i] = y[i] / s;
    mpr(NDIM,a,x,n,y);
    inpr(x,y,n,&enew);
    error = fabs((enew-eval)/enew);
    eval = enew;
    niter++;
    printf("(NITER,EVAL,ERROR=%d %g %g\n", niter,eval,error);
    if (niter == m) break;
  }
  while (error > eps);
  // OUTPUT
  printf("NO. OF ITERATIONS=%3d   EPS=%12.4e\n", niter, eps);
  printf("EIGENVALUE=%12.4e\n", eval);
  printf("** EIGENVECTOR **\n");
  for (i = 0; i < 5; i++)
    printf("  I     X(I)    ");
  printf("\n");
  for (i=0; i<n; i++) {
    printf("%3d%12.4e ", i+1, x[i]);
    if ((i+1) % 5 == 0 || (i+1) == n)
      printf("\n");
  }
}

void mpr(int ndim, double a[][ndim], double x[], int n, double y[])
{
  int i, j;
  // MATRIX MULTIPLICATION
  for (i=0; i<n; i++) {
    y[i] =0.0;
    for (j=0; j<n; j++)
      y[i] += a[i][j] * x[j];
  }
}

void inpr(double x[], double y[], int n, double *s)
{
  int i;
  // INNER PRODUCT
  *s = 0.0;
  for (i = 0; i < n; i++)
    *s += x[i] * y[i];
}
