// 菊地文雄, 山本昌宏「微分方程式と計算機演習」, 山海堂 (1991), p. 98
/* ** INVERSE ITERATION METHOD **
   N : ORDER OF MATRIX
   A : COEFFICIENT MATRIX
   EPS : THRESHOLD VALUE FOR ERROR
   M : MAXIMUM NO. OF ITERATIONS
   EVAL : EIGENVALUE
   X : EIGENVECTOR
   NITER : NO. OF ITERATIONS
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void gauss(int ndim, double a[][ndim], double f[], int N, int ID);
void inpr(double x[], double y[], int n, double *s);

#define NDIM (100)

int main(void)
{
  int n, m, i, j, niter;
  double eps, eval, s, t, enew, error;
  double a[NDIM][NDIM], x[NDIM], y[NDIM];
  // INPUT
  printf("INPUT : N= \n");
  scanf("%d", &n);
  if (n <= 1) exit(1);
  printf("INPUT: EPS=\n");
  scanf("%lf", &eps);
  printf("INPUT : M= \n");
  scanf("%d", &m);
  for (i = 0; i < n; i++) {
    for (j=0; j < n; j++) {
      printf("INPUT : A(%3d,%3d)=\n", i+1, j+1);
      scanf("%lf", &a[i][j]);
    }
  }
  for (i=0; i<n; i++) {
    printf("INPUT:X(%3d)=\n",i+1);
    scanf("%lf", &x[i]);
  }
  // ITERATION
  niter = 0;
  eval = 0.0;
  do {
    for (i=0; i<n; i++)
      y[i] = x[i];
    gauss(NDIM,a,y,n,niter);
    inpr(x,y,n,&s);
    inpr(y,y,n,&t);
    enew = s / t;
    t = sqrt(t);
    for (i=0; i<n; i++)
      x[i] = y[i] / t;
    error = fabs((enew-eval)/enew);
    eval = enew;
    niter++;
    printf("(NITER,EVAL,ERROR)=%d %g %g\n", niter, eval, error);
    if (niter == m) break;
  }
  while (error > eps);
  // OUTPUT
  printf("NO. OF ITERATIONS = %3d   EPS=%12.4e\n", niter, eps);
  printf("EIGENVALUE=%12.4e\n", eval);
  printf("**EIGENVECTOR**\n");
  for (i = 0; i < 5; i++)
    printf("  I     X(I)    ");
  printf("\n");
  for (i=0; i<n; i++) {
    printf("%3d%12.4e ", i+1, x[i]);
    if ((i+1) % 5 == 0 || (i+1) == n)
      printf("\n");
  }
}

void gauss(int ndim, double a[][ndim], double f[], int n, int id)
{
  int i, j, k;
  double aa;
  // FORWARD ELIMINATION
  for (i=0; i<n-1; i++) {
    for (j=i+1; j<n; j++) {
      aa = a[j][i] / a[i][i];
      if (id == 0) {
    for (k=i+1; k<n; k++)
      a[j][k] -= aa * a[i][k];
      }
      f[j] -= aa * f[i];
    }
  }
  // BACKWARD SUBSTITUTION
  f[n-1] /= a[n-1][n-1];
  for (i=n-2; i>= 0; i--) {
    for (j=i+1; j<n; j++)
      f[i] -= a[i][j] * f[j];
    f[i] /= a[i][i];
  }
}

void inpr(double x[], double y[], int n, double *s)
{
  int i;
  // INNER PRODUCT
  *s = 0.0;
  for (i=0; i<n; i++)
    *s += x[i] * y[i];
}
